package nl.sbdeveloper.mctpaudio;

import co.aikar.commands.PaperCommandManager;
import lombok.Getter;
import nl.sbdeveloper.mctpaudio.commands.TPAudioCMD;
import nl.sbdeveloper.mctpaudio.listener.LogoutListener;
import nl.sbdeveloper.mctpaudio.listener.WGListener;
import nl.sbdeveloper.mctpaudio.managers.WGManager;
import nl.sbdeveloper.mctpaudio.radio.Playlist;
import nl.sbdeveloper.mctpaudio.socket.Client;
import org.bukkit.Bukkit;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.java.JavaPlugin;

public final class ThemeParkAudio extends JavaPlugin {
    @Getter
    private static Plugin instance;

    @Getter
    private static PaperCommandManager commandManager;

    @Getter
    private static Client client;

    @Getter
    private static Playlist playlist;

    public void onEnable() {
        Bukkit.getLogger().info("[ThemeParkAudio] Loading...");

        if (!setupPlugins()) {
            Bukkit.getLogger().severe("[ThemeParkAudio] WorldGuard not found! Disabling...");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        instance = this;

        getConfig().options().copyDefaults(true);
        saveDefaultConfig();

        Bukkit.getLogger().info("[ThemeParkAudio] Connecting with socket...");
        client = new Client(getConfig().getString("socketUrl"));
        client.connect();

        Bukkit.getLogger().info("[ThemeParkAudio] Loading commands and events...");
        commandManager = new PaperCommandManager(this);
        commandManager.enableUnstableAPI("help");

        commandManager.registerCommand(new TPAudioCMD());

        Bukkit.getPluginManager().registerEvents(new WGListener(), this);
        Bukkit.getPluginManager().registerEvents(new LogoutListener(), this);

        Bukkit.getLogger().info("[ThemeParkAudio] Loading playlist...");
        playlist = new Playlist();

        Bukkit.getLogger().info("[ThemeParkAudio] Plugin is enabled!");
    }

    public void onDisable() {
        client.disconnect();
        instance = null;
        Bukkit.getLogger().info("[ThemeParkAudio] Plugin is disabled!");
    }

    private boolean setupPlugins() {
        if (Bukkit.getPluginManager().getPlugin("WorldGuard") != null) {
            WGManager.setWorldGuard(getServer().getPluginManager().getPlugin("WorldGuard"));
            return true;
        }
        return false;
    }
}
