package nl.sbdeveloper.mctpaudio.api.maps;

import java.util.ArrayList;
import java.util.Random;

/**
 * An {@link ArrayList} with shuffle support.
 *
 * @param <E> The type you want to store
 */
public class SongList<E> extends ArrayList<E> {
    private final Random r = new Random();

    /**
     * Shuffle this List
     */
    public void shuffle() {
        for (int index = 0; index < size(); index++) {
            int secondIndex = r.nextInt(size());
            swap(index, secondIndex);
        }
    }

    /**
     * Get a random item from this List
     *
     * @return The random element, or null if empty
     */
    public E getRandom() {
        int size = size();
        if (size == 0) return null;
        return get(r.nextInt(size()));
    }

    /**
     * Swaps two elements in an {@link ArrayList}
     *
     * @param firstIndex The first index
     * @param secondIndex The second index
     */
    private void swap(int firstIndex, int secondIndex) {
        E first = get(firstIndex);
        E second = get(secondIndex);

        set(secondIndex, first);
        set(firstIndex, second);
    }
}