package nl.sbdeveloper.mctpaudio.listener;

import com.bergerkiller.bukkit.common.entity.CommonEntityController;
import com.bergerkiller.bukkit.common.events.EntityMoveEvent;
import com.bergerkiller.bukkit.tc.controller.MinecartGroup;
import com.bergerkiller.bukkit.tc.controller.MinecartGroupStore;
import com.bergerkiller.bukkit.tc.controller.MinecartMember;
import com.bergerkiller.bukkit.tc.controller.MinecartMemberStore;
import com.sk89q.worldguard.protection.regions.ProtectedRegion;
import net.raidstone.wgevents.events.RegionsEnteredEvent;
import nl.sbdeveloper.mctpaudio.ThemeParkAudio;
import nl.sbdeveloper.mctpaudio.api.AudioType;
import nl.sbdeveloper.mctpaudio.managers.PinManager;
import nl.sbdeveloper.mctpaudio.managers.WGManager;
import nl.sbdeveloper.mctpaudio.socket.messages.AudioMessage;
import nl.sbdeveloper.mctpaudio.socket.messages.LightMessage;
import nl.sbdeveloper.mctpaudio.socket.messages.StopAudioMessage;
import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerTeleportEvent;

import java.util.*;

public class WGListener implements Listener {
    /**
     * Music detection
     */
    @EventHandler
    public void onPlayerMove(PlayerMoveEvent e) {
        handlePlayerMovement(e.getPlayer(), e.getFrom(), e.getTo());
    }

    @EventHandler
    public void onPlayerTeleport(PlayerTeleportEvent e) {
        handlePlayerMovement(e.getPlayer(), e.getFrom(), e.getTo());
    }

    @EventHandler
    public void onEntityMove(EntityMoveEvent e) {
        MinecartGroup group = MinecartGroupStore.get(e.getEntity());
        MinecartMember<?> member = MinecartMemberStore.getFromEntity(e.getEntity());

        if (group != null && group.stream().map(gMember -> gMember.getEntity().getPlayerPassengers()).mapToLong(Collection::size).sum() != 0) {
            group.stream().map(CommonEntityController::getEntity).forEach(ent ->
                    ent.getPlayerPassengers().forEach(p -> handlePlayerMovement(p, ent.getLastLocation(), ent.getLocation())));
        } else if (member != null && !member.getEntity().getPlayerPassengers().isEmpty()) {
            member.getEntity().getPlayerPassengers().forEach(p ->
                    handlePlayerMovement(p, member.getEntity().getLastLocation(), member.getEntity().getLocation()));
        }
    }

    private void handlePlayerMovement(Player player, Location from, Location to) {
        if (to == null || from.getWorld() == null || to.getWorld() == null) return;

        if (from.getBlockX() != to.getBlockX() || from.getBlockY() != to.getBlockY() || from.getBlockZ() != to.getBlockZ()) {
            if (!PinManager.hasPin(player.getUniqueId())) return;

            Set<String> list = ThemeParkAudio.getPlugin().getConfig().getConfigurationSection("regions.audio").getKeys(false);

            List<String> fromRegions = WGManager.getRegionsIn(from).stream().map(ProtectedRegion::getId).toList();
            List<String> toRegions = WGManager.getRegionsIn(to).stream().map(ProtectedRegion::getId).toList();

            if ((Collections.disjoint(list, fromRegions) && !Collections.disjoint(list, toRegions)) || (!Collections.disjoint(list, fromRegions) && !Collections.disjoint(list, toRegions))) {
                //Walked in a region
                boolean found = false;
                for (Entity ent : to.getChunk().getEntities()) {
                    MinecartMember<?> member = MinecartMemberStore.getFromEntity(ent);
                    if (member == null) continue;
                    if (member.getEntity().getPlayerPassengers().contains(player)) {
                        found = true;
                        break;
                    }
                }

                if (found) return; //The player is inside a vehicle, ignore...

                if (!Collections.disjoint(list, fromRegions) && !Collections.disjoint(list, toRegions)) {
                    Optional<String> name = fromRegions.stream().filter(list::contains).findFirst();
                    Optional<String> name2 = toRegions.stream().filter(list::contains).findFirst();

                    if (name.isPresent() && name2.isPresent() && name.get().equals(name2.get()))
                        return; //Beide heeft een region, en dat is dezelfde.

                    if (name.isPresent() && name2.isPresent()) {
                        String regionURL = ThemeParkAudio.getPlugin().getConfig().getString("regions.audio." + name.get());
                        String regionURL2 = ThemeParkAudio.getPlugin().getConfig().getString("regions.audio." + name2.get());

                        if (regionURL.equals(regionURL2))
                            return; //Beide heeft een region, niet dezelfde, maar wel met dezelfde muziek.
                    }
                }

                Optional<String> name = toRegions.stream().filter(list::contains).findFirst();
                if (name.isEmpty()) return;
                String regionURL = ThemeParkAudio.getPlugin().getConfig().getString("regions.audio." + name.get());

                AudioMessage.of(AudioType.MUSIC, regionURL).send(player.getUniqueId());
            } else if (!Collections.disjoint(list, fromRegions) && Collections.disjoint(list, toRegions)) {
                //Not in a region, stop...
                StopAudioMessage.of(AudioType.MUSIC).send(player.getUniqueId());
            }
        }
    }

    /**
     * Hue detection
     */
    @EventHandler
    public void onRegionEnter(RegionsEnteredEvent e) {
        if (e.getPlayer() == null) return;

        Set<String> list2 = ThemeParkAudio.getPlugin().getConfig().getConfigurationSection("regions.light").getKeys(false);

        if (!Collections.disjoint(list2, e.getRegionsNames())) {
            //One element is the same -> In a region

            Optional<String> name = e.getRegionsNames().stream().filter(list2::contains).findFirst();
            if (name.isEmpty()) return;
            String configData = ThemeParkAudio.getPlugin().getConfig().getString("regions.light." + name.get());

            if (configData != null) {
                LightMessage.of(configData).send(e.getPlayer().getUniqueId());
            }
        }
    }
}
