package nl.sbdeveloper.mctpaudio.socket.messages;

import lombok.Getter;
import nl.sbdeveloper.mctpaudio.ThemeParkAudio;
import org.bukkit.Bukkit;
import org.bukkit.entity.Entity;
import org.jetbrains.annotations.Nullable;
import org.json.simple.JSONObject;

import java.util.List;
import java.util.UUID;
import java.util.function.Function;

@Getter
public abstract class AbstractMessage {
    private final MessageTask task;

    public AbstractMessage(MessageTask task) {
        this.task = task;
    }

    public void send(UUID uuid) {
        send(List.of(uuid), null);
    }

    public void send(List<UUID> uuids) {
        send(uuids, null);
    }

    public void broadcast(@Nullable Function<UUID, Boolean> filter) {
        send(Bukkit.getOnlinePlayers().stream().map(Entity::getUniqueId).toList(), filter);
    }

    public void send(List<UUID> uuids, @Nullable Function<UUID, Boolean> filter) {
        JSONObject data = new JSONObject();
        data.put("task", getTask());
        JSONObject extendData = extendJson();
        if (extendData != null) {
            data.putAll(extendData);
        }

        for (UUID uuid : uuids) {
            if (filter == null || filter.apply(uuid)) {
                data.put("uuid", uuid.toString().replace("-", ""));
                ThemeParkAudio.getClient().sendData(data);
            }
        }
    }

    protected abstract JSONObject extendJson();
}