package nl.sbdeveloper.mctpaudio.socket.messages;

import nl.sbdeveloper.mctpaudio.api.LightRegion;
import org.jetbrains.annotations.NotNull;
import org.json.simple.JSONObject;

public class LightMessage extends AbstractMessage {
    private final int brightness;
    private final int r, g, b, w;
    private final LightRegion region;

    private LightMessage(int brightness, int r, int g, int b, int w, LightRegion region) {
        super(MessageTask.LIGHT);
        this.brightness = brightness;
        this.r = r;
        this.g = g;
        this.b = b;
        this.w = w;
        this.region = region;
    }

    public static LightMessage of(int brightness, int r, int g, int b, int w, LightRegion region) {
        return new LightMessage(brightness, r, g, b, w, region);
    }

    private LightMessage(@NotNull String configData) {
        super(MessageTask.LIGHT);

        String[] configDataSplit = configData.split("_");
        if (configDataSplit.length != 6) {
            throw new IllegalArgumentException("Invalid config data: " + configData);
        }

        this.brightness = Integer.parseInt(configDataSplit[0]);
        this.r = Integer.parseInt(configDataSplit[1]);
        this.g = Integer.parseInt(configDataSplit[2]);
        this.b = Integer.parseInt(configDataSplit[3]);
        this.w = Integer.parseInt(configDataSplit[4]);
        this.region = LightRegion.valueOf(configDataSplit[5]);
    }

    public static LightMessage of(@NotNull String configData) {
        return new LightMessage(configData);
    }

    @Override
    protected JSONObject extendJson() {
        JSONObject data = new JSONObject();
        data.put("rgbw", r + "_" + g + "_" + b + "_" + w);
        data.put("region", region);
        data.put("brightness", brightness);
        return data;
    }
}